using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bricksoft;
using Bricksoft.IM.AIM;

namespace Bricksoft.IM.AIMCF.Demo
{
    public partial class frmAIMCF : Form
    {
        public delegate void MyInvoke(string str);
        
        private TreeNode rootNode;

        private AIMClient myAIM = new AIMClient();

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmAIMCF()
        {
            InitializeComponent();

            myAIM.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            myAIM.OnError += delegate(object sender, string error) { this.Invoke(new OnErrorEventHandler(OnError), new object[] { sender, error }); };
            myAIM.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            myAIM.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            myAIM.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            myAIM.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            myAIM.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            myAIM.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            myAIM.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            myAIM.OnMessageSent += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessageSent), new object[] { sender, buddy, message }); };
        }

        private int statusAsImageIndex(AIMStatus s)
        {
            switch (s)
            {
                case AIMStatus.aimOnline:
                    return 1;                  
                case AIMStatus.aimAway:
                    return 2;
                default:
                    return 3;
            }
        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageIndex = statusAsImageIndex(myAIM.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            AIMBuddy yb = (AIMBuddy)myAIM.BuddyList[buddy];

            ((TreeNode)yb.Tag).ImageIndex = statusAsImageIndex(yb.Status);
            ((TreeNode)yb.Tag).SelectedImageIndex = ((TreeNode)yb.Tag).ImageIndex;

            ((TreeNode)yb.Tag).Text = yb.Account;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(myAIM.Account);
            rootNode.ImageIndex = statusAsImageIndex(myAIM.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;

            loginToolStripMenuItem.Enabled = false;
            cmdLogin.Enabled = false;
            logoutToolStripMenuItem.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = myAIM.Account;

            tabControl1.SelectedIndex = 1;

            lblStatus.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            loginToolStripMenuItem.Enabled = true;
            cmdLogin.Enabled = true;

            logoutToolStripMenuItem.Enabled = false;

            tabControl1.SelectedIndex = 0;

            lblStatus.Text = "Logout.";
        }

        private TreeNode FindNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageIndex == 0) && ((tnParent.Text == strValue)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in myAIM.BuddyList)
            {
                AIMBuddy jb = (AIMBuddy)buddy.Value;

                TreeNode gn = null;

                if (myAIM.GroupList.ContainsKey(jb.GroupID))
                {
                    OSCARGroup group = myAIM.GroupList[jb.GroupID] as OSCARGroup;
                    gn = FindNode(rootNode, group.GroupName);
                }

                TreeNode bn = new TreeNode(jb.Account);
                bn.ImageIndex = statusAsImageIndex(jb.Status);
                bn.SelectedImageIndex = bn.ImageIndex;
                bn.Tag = jb;
                jb.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                {
                    gn.Nodes.Add(bn);
                    gn.Expand();
                }
            }

            rootNode.ExpandAll();

            showLog("Please select a contact to send message");
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in myAIM.GroupList)
            {
                OSCARGroup g = (OSCARGroup)group.Value;

                TreeNode gn = new TreeNode(g.GroupName);
                rootNode.Nodes.Add(gn);
                gn.ImageIndex = 0;
                gn.SelectedImageIndex = gn.ImageIndex;
            }

            rootNode.ExpandAll();
        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog("OnMessage --> " + buddy + " says: " + message);

            lblStatus.Text = buddy + " says: " + message;
        }

        private void OnMessageSent(object sender, string buddy, string message)
        {
            showLog("OnMessageSent --> says to (" + buddy + "): " + message);
        }

        private void OnLog(object sender, string log)
        {
            showLog("OnLog --> " + log);
            lblStatus.Text = log;
        }

        private void OnError(object sender, string Error)
        {
            showLog("OnError -> " + Error);

            lblStatus.Text = Error;

            if (!myAIM.Logined)
                cmdLogin.Enabled = true;
        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            cmdLogin.Enabled = false;

            myAIM.Account = txtEmail.Text;
            myAIM.Password = txtPassword.Text;
            myAIM.Login();
        }

        private void tvIM_AfterSelect(object sender, TreeViewEventArgs e)
        {
            mmAction.Enabled = tvIM.SelectedNode.Tag is AIMBuddy;
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is AIMBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    myAIM.SendMessage((tvIM.SelectedNode.Tag as AIMBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            myAIM.Logout();
        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin.Enabled = false;

            myAIM.Account = txtEmail.Text;
            myAIM.Password = txtPassword.Text;
            myAIM.Login();
        }
    }
}