using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bricksoft;
using Bricksoft.IM.ICQ;

namespace Bricksoft.IM.ICQCF.Demo
{
    public partial class frmICQCF : Form
    {
        public delegate void MyInvoke(string str);
        
        private TreeNode rootNode;
       
        private ICQClient myICQ = new ICQClient();

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmICQCF()
        {
            InitializeComponent();

            myICQ.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            myICQ.OnError += delegate(object sender, string error) { this.Invoke(new OnErrorEventHandler(OnError), new object[] { sender, error }); };
            myICQ.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            myICQ.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            myICQ.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            myICQ.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            myICQ.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            myICQ.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            myICQ.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            myICQ.OnMessageSent += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessageSent), new object[] { sender, buddy, message }); };
        }

        private int statusAsImageIndex(ICQStatus s)
        {
            switch (s)
            {
                case ICQStatus.icqNA:
                    return 3;
                case ICQStatus.icqAway:
                    return 1;
                case ICQStatus.icqOccupied:
                    return 4;
                default:
                    return 2;
                case ICQStatus.icqOnline:
                    return 5;
            }

        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageIndex = statusAsImageIndex(myICQ.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            ICQBuddy yb = (ICQBuddy)myICQ.BuddyList[buddy];

            ((TreeNode)yb.Tag).ImageIndex = statusAsImageIndex(yb.Status);
            ((TreeNode)yb.Tag).SelectedImageIndex = ((TreeNode)yb.Tag).ImageIndex;

            ((TreeNode)yb.Tag).Text = yb.Account;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(myICQ.Account);
            rootNode.ImageIndex = statusAsImageIndex(myICQ.Status);
            rootNode.SelectedImageIndex = rootNode.ImageIndex;

            loginToolStripMenuItem.Enabled = false;
            cmdLogin.Enabled = false;
            logoutToolStripMenuItem.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = myICQ.Account;

            tabControl1.SelectedIndex = 1;

            lblStatus.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            loginToolStripMenuItem.Enabled = true;
            cmdLogin.Enabled = true;

            logoutToolStripMenuItem.Enabled = false;

            tabControl1.SelectedIndex = 0;

            lblStatus.Text = "Logout.";
        }

        private TreeNode FindNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageIndex == 0) && ((tnParent.Text == strValue)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in myICQ.BuddyList)
            {
                ICQBuddy jb = (ICQBuddy)buddy.Value;

                TreeNode gn = null;

                if (myICQ.GroupList.ContainsKey(jb.GroupID))
                {
                    OSCARGroup group = myICQ.GroupList[jb.GroupID] as OSCARGroup;
                    gn = FindNode(rootNode, group.GroupName);
                }

                TreeNode bn = new TreeNode(jb.Account);
                bn.ImageIndex = statusAsImageIndex(jb.Status);
                bn.SelectedImageIndex = bn.ImageIndex;
                bn.Tag = jb;
                jb.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                {
                    gn.Nodes.Add(bn);
                    gn.Expand();
                }
            }

            rootNode.ExpandAll();

            showLog("Please select a contact to send message");
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in myICQ.GroupList)
            {
                OSCARGroup g = (OSCARGroup)group.Value;

                TreeNode gn = new TreeNode(g.GroupName);
                rootNode.Nodes.Add(gn);
                gn.ImageIndex = 0;
                gn.SelectedImageIndex = gn.ImageIndex;
            }

            rootNode.ExpandAll();
        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog("OnMessage --> " + buddy + " says: " + message);

            lblStatus.Text = buddy + " says: " + message;
        }

        private void OnMessageSent(object sender, string buddy, string message)
        {
            showLog("OnMessageSent --> says to (" + buddy + "): " + message);
        }

        private void OnLog(object sender, string log)
        {
            showLog("OnLog --> " + log);
            lblStatus.Text = log;
        }

        private void OnError(object sender, string Error)
        {
            showLog("OnError -> " + Error);

            lblStatus.Text = Error;

            if (!myICQ.Logined)
                cmdLogin.Enabled = true;
        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            cmdLogin.Enabled = false;

            myICQ.Account = txtEmail.Text;
            myICQ.Password = txtPassword.Text;
            myICQ.Login();
        }

        private void tvIM_AfterSelect(object sender, TreeViewEventArgs e)
        {
            mmAction.Enabled = tvIM.SelectedNode.Tag is ICQBuddy;
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is ICQBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    myICQ.SendMessage((tvIM.SelectedNode.Tag as ICQBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            myICQ.Logout();
        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin.Enabled = false;

            myICQ.Account = txtEmail.Text;
            myICQ.Password = txtPassword.Text;
            myICQ.Login();
        }
    }
}