﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using Bricksoft;
using Bricksoft.IM.Jabber;

namespace JabberDemo
{
    public partial class frmRobot : Form
    {
        private JabberClient myJabber = new JabberClient();

        private TreeNode rootNode = null;
        public delegate void MyInvoke(string str);

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmRobot()
        {
            InitializeComponent();
          
            //public enum JabberStatus {jsOffline, jsAway, jsOnline, jsCustom, jsInvisible};

            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("Online");
            cbStatus.Items.Add("Custom");
            cbStatus.Items.Add("Invisible");

            myJabber.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            myJabber.OnError += delegate(object sender, string error) { this.Invoke(new OnErrorEventHandler(OnError), new object[] { sender, error }); };
            myJabber.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            myJabber.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            myJabber.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            myJabber.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            myJabber.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            myJabber.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            myJabber.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            myJabber.OnMessageSent += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessageSent), new object[] { sender, buddy, message }); };
            myJabber.OnAddBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnAddBuddy), new object[] { sender, buddy }); };
            myJabber.OnDeleteBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnDeleteBuddy), new object[] { sender, buddy }); };
            myJabber.OnBuddyAddYouRequest += delegate(object sender, string buddy, ref bool bAccept)
            {
                object[] inVar = { sender, buddy, bAccept };
                this.Invoke(new OnBuddyAddYouRequestEventHandler(OnBuddyAddYouRequest), inVar);
                bAccept = (bool)inVar[2];
            };
        }

        private void OnBuddyAddYouRequest(object sender, string buddy, ref bool accept)
        {
            if (MessageBox.Show("\""+ buddy + "\" added you. Do you want to \"" + buddy + "\" to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                accept = true;
            }
            else
                accept = false;
        }

        private string statusAsImageKey(JabberStatus status)
        {
            switch (status)
            {

                case JabberStatus.jsOnline: return "available.png";
                case JabberStatus.jsAway: return "away.png";
                default: return "offline.png";                  
            }
        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageKey = statusAsImageKey(myJabber.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;

            cbStatus.SelectedIndex = (int)myJabber.Status - 1;
        }

        private void OnAddBuddy(object sender, string buddy)
        {
            JabberBuddy jb = (JabberBuddy)myJabber.BuddyList[buddy];

            TreeNode gn = FindNode(rootNode, jb.Group);

            TreeNode bn = new TreeNode(jb.Account);
            bn.ImageKey = statusAsImageKey(jb.Status);
            bn.SelectedImageKey = bn.ImageKey;
            bn.Tag = jb;
            jb.Tag = bn;

            if (gn == null)
                rootNode.Nodes.Add(bn);
            else
                gn.Nodes.Add(bn);

            rootNode.ExpandAll();
        }

        private void OnDeleteBuddy(object sender, string buddy)
        {
            JabberBuddy jb = (JabberBuddy)myJabber.BuddyList[buddy];

            (jb.Tag as TreeNode).Remove();
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            JabberBuddy yb = (JabberBuddy)myJabber.BuddyList[buddy];

            ((TreeNode)yb.Tag).ImageKey = statusAsImageKey(yb.Status);
            ((TreeNode)yb.Tag).SelectedImageKey = ((TreeNode)yb.Tag).ImageKey;

            if ((yb.StatusMessage != null) && (yb.StatusMessage != ""))
                ((TreeNode)yb.Tag).Text = yb.Account + " - " + yb.StatusMessage;
            else
                ((TreeNode)yb.Tag).Text = yb.Account;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(myJabber.Account + "@" + myJabber.Server);
            rootNode.ImageKey = statusAsImageKey(myJabber.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;

            loginToolStripMenuItem.Enabled = false;
            cmdLogin.Enabled = false;
            logoutToolStripMenuItem.Enabled = true;
            addBuddyToolStripMenuItem.Enabled = true;

            cbStatus.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = myJabber.Account + "@" + myJabber.Server;

            tabControl1.SelectedIndex = 1;

            lblStatus.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            loginToolStripMenuItem.Enabled = true;
            cmdLogin.Enabled = true;
            addBuddyToolStripMenuItem.Enabled = false;

            logoutToolStripMenuItem.Enabled = false;

            cbStatus.Enabled = false;

            tabControl1.SelectedIndex = 0;

            lblStatus.Text = "Logout.";

        }

        private TreeNode FindNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageKey == "group.png") && ((tnParent.Text == strValue)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in myJabber.BuddyList)
            {
                JabberBuddy jb = (JabberBuddy)buddy.Value;

                TreeNode gn = FindNode(rootNode, jb.Group);

                TreeNode bn = new TreeNode(jb.Account);
                bn.ImageKey = statusAsImageKey(jb.Status);
                bn.SelectedImageKey = bn.ImageKey;
                bn.Tag = jb;
                jb.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                {
                    gn.Nodes.Add(bn);
                    gn.Expand();
                }
            }

            rootNode.ExpandAll();

            showLog("Please select a contact to send message");
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in myJabber.GroupList)
            {
                string g = (string)group.Value;

                TreeNode gn = new TreeNode(g);
                rootNode.Nodes.Add(gn);
                gn.ImageKey = "group.png";
                gn.SelectedImageKey = gn.ImageKey;
            }

            rootNode.ExpandAll();
        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog("OnMessage --> " + buddy + " says: " + message);
        }

        private void OnMessageSent(object sender, string buddy, string message)
        {
            showLog("OnMessageSent --> says to (" + buddy + "): " + message);
        }

        private void OnLog(object sender, string log)
        {
            showLog("OnLog --> " + log);
            lblStatus.Text = log;
        }

        private void OnError(object sender, string Error)
        {
            showLog("OnError -> " + Error);
            lblStatus.Text = Error;

            if (!myJabber.Logined)
                cmdLogin.Enabled = true;
        }

        private void cbServer_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbServer.SelectedIndex == 1)
            {
                txtConnectServer.Text = "talk.google.com";
                cbAuth.SelectedIndex = 3;
            }
            else
            {
                txtConnectServer.Text = "";
                cbAuth.SelectedIndex = 0;
            }
        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            myJabber.Status = (JabberStatus)Enum.ToObject(typeof(JabberStatus), cbStatus.SelectedIndex + 1);
        }


        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin.Enabled = false;

            myJabber.Account = txtAccount.Text;
            myJabber.Password = txtPassword.Text;
            myJabber.Server = cbServer.Text;
            myJabber.ConnectServer = txtConnectServer.Text;
            myJabber.ServerPort = Convert.ToInt32(txtPort.Text);
            myJabber.ClientAuthMetod = (JabberAuthMethod)Enum.ToObject(typeof(JabberAuthMethod), cbAuth.SelectedIndex);

            myJabber.Login();
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            myJabber.Logout();
        }

        private void tvIM_AfterSelect(object sender, TreeViewEventArgs e)
        {
            mmAction.Enabled = tvIM.SelectedNode.Tag is JabberBuddy;
        }

        private void sendMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MsgForm mf = new MsgForm();

            mf.ShowDialog();

            if ((mf.ModalResult == DialogResult.OK) && (mf.Message != ""))
            {
                myJabber.SendMessage((tvIM.SelectedNode.Tag as JabberBuddy).Account, mf.Message);
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            myJabber.DeleteBuddy((tvIM.SelectedNode.Tag as JabberBuddy).Account);
        }

        private void addBuddyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MsgForm mf = new MsgForm();

            mf.Set("Add...","Buddy:");

            mf.ShowDialog();

            if ((mf.ModalResult == DialogResult.OK) && (mf.Message != ""))
            {
                myJabber.AddBuddy(mf.Message);
            }
        }


    }
}