using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Bricksoft;
using Bricksoft.IM.MSN;

namespace Bricksoft.IM.MSNCF.Demo
{
    public partial class frmMSNCF : Form
    {
        private MSNClient msn = new MSNClient();
        private TreeNode rootNode;

        public frmMSNCF()
        {
            InitializeComponent();

            msn.OnLog += MSN_OnLog;
            msn.OnLogin += MSN_OnLogin;
            msn.OnLogout += MSN_OnLogout;
            msn.OnReceiveBuddyList += MSN_OnReceiveBuddyList;
            msn.OnReceiveGroupList += MSN_OnReceiveGroupList;
            msn.OnBuddyStatusChanged += MSN_OnBuddyStatusChanged;
            msn.OnStatusChanged += MSN_OnStatusChanged;
            msn.OnTextMessageReceived += MSN_OnTextMessageReceived;
            msn.OnError += MSN_OnError;
            msn.OnNicknameChanged += MSN_OnNicknameChanged;
            msn.OnOfflineMessageReceived += MSN_OnOfflineMessageReceived;
       }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void OnOfflineMessageReceived(object sender, string buddy, string message,DateTime receivedTime)
        {
            txtLog.Text = buddy + " Sent " + receivedTime.ToLocalTime() + ": " + message + "\r\n" + txtLog.Text;
            sbar.Text = "Receive offline message from " + buddy;
        }

        private void MSN_OnOfflineMessageReceived(object sender, string buddy, string message,DateTime receivedTime)
        {
            this.Invoke(new OnOfflineMessageEventHandler(OnOfflineMessageReceived), new object[] { sender, buddy, message,receivedTime});
        } 

        private void OnNicknameChanged(object sender, EventArgs e)
        {
            rootNode.Text = ((MSNClient)sender).Nickname;
            txtNickname.Text = ((MSNClient)sender).Nickname;
        }

        private void MSN_OnNicknameChanged(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnNicknameChanged), new object[] { sender, e });
        }

        private void OnLog(object sender, string statusMessage)
        {
            if (statusMessage.Length < 50)
                sbar.Text = statusMessage;
            else
                sbar.Text = statusMessage.Substring(0, 50);

            txtLog.Text = statusMessage + "\r\n" + txtLog.Text;
        }

        private void MSN_OnLog(object sender, string log)
        {
            this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log });
        }

        private void OnReceiveGroupList(Object sender, EventArgs e)
        {
            MSNClient inMSN = (MSNClient)sender;
            foreach (System.Collections.DictionaryEntry o in inMSN.GroupList)
            {
                MSNGroup g = (MSNGroup)o.Value;

                TreeNode gn = new TreeNode(g.GroupName);
                rootNode.Nodes.Add(gn);
                gn.SelectedImageIndex = 0;
                gn.ImageIndex = 0;
                gn.Tag = g;
                g.Tag = gn;
            }
            rootNode.ExpandAll();
        }

        private void MSN_OnReceiveGroupList(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnReceiveGroupList), new object[] { sender, e });
        }

        private void MSN_OnReceiveBuddyList(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnReceiveBuddyList), new object[] { sender, e });
        }

        private TreeNode FindNode(TreeNode tnParent, string strValue)
        {

            if (tnParent == null) return null;

            if ((tnParent.Tag is MSNGroup) && ((tnParent.Tag as MSNGroup).GroupID == strValue))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, strValue);
                if (tnRet != null) break;
            }
            return tnRet;
        }

        private int MSNStatusToImageIndex(MSNStatus status)
        {
            switch (status)
            {
                case MSNStatus.msnIdle:
                case MSNStatus.msnAway:
                case MSNStatus.msnBeRightBack:
                case MSNStatus.msnOutLunch:
                    return 3;

                case MSNStatus.msnOnline:
                    return 4;

                case MSNStatus.msnBusy:
                case MSNStatus.msnOnThePhone:
                    return 1;

                default:
                    return 5;
            }
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            MSNBuddy b = msn.BuddyList[buddy] as MSNBuddy;
            (b.Tag as TreeNode).Text = b.Nickname;


            if (b.Blocked)
            {
                if (b.Status == MSNStatus.msnAppearOffline)
                {
                    (b.Tag as TreeNode).ImageIndex = 6;
                    (b.Tag as TreeNode).SelectedImageIndex = 6;
                }
                else
                {
                    (b.Tag as TreeNode).ImageIndex = 2;
                    (b.Tag as TreeNode).SelectedImageIndex = 2;
                }
            }
            else
            {
                (b.Tag as TreeNode).ImageIndex = MSNStatusToImageIndex(b.Status);
                (b.Tag as TreeNode).SelectedImageIndex = MSNStatusToImageIndex(b.Status);
            }
        }

        private void MSN_OnBuddyStatusChanged(object sender, string buddy)
        {
            this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy });
        }

        private void OnReceiveBuddyList(Object sender, EventArgs e)
        {
            MSNClient inMSN = (MSNClient)sender;
            foreach (System.Collections.DictionaryEntry o in inMSN.BuddyList)
            {
                MSNBuddy b = (MSNBuddy)o.Value;

                TreeNode gn = FindNode(rootNode, b.GroupID);

                TreeNode bn = new TreeNode(b.Nickname);
                bn.ImageIndex = 5;
                bn.SelectedImageIndex = 5;
                bn.Tag = b;
                b.Tag = bn;

                if (gn == null)
                    rootNode.Nodes.Add(bn);
                else
                    gn.Nodes.Add(bn);
            }
            rootNode.ExpandAll();
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(msn.Account);

            cmdLogin.Enabled = false;
            cbStatus.Enabled = true;
            txtNickname.Enabled = true;

            tvMSN.Nodes.Add(rootNode);

            rootNode.Text = msn.Account;
            rootNode.ImageIndex = 5;
            rootNode.SelectedImageIndex = 5;

            tabControl.SelectedIndex = 1;

            sbar.Text = "Login OK.";
        }


        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageIndex = MSNStatusToImageIndex(((MSNClient)sender).Status);
            rootNode.SelectedImageIndex = MSNStatusToImageIndex(((MSNClient)sender).Status);
            cbStatus.SelectedIndex = (int)((MSNClient)sender).Status;
        }

        private void MSN_OnStatusChanged(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e });
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvMSN.Nodes.Clear();

            txtNickname.Enabled = false;
            cmdLogin.Enabled = true;
            cbStatus.Enabled = false;
            mmAction.Enabled = false;

            tabControl.SelectedIndex = 0;

            sbar.Text = "Logout.";
        }

        private void MSN_OnLogin(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnLogin));
        }

        private void MSN_OnLogout(object sender, EventArgs e)
        {
            this.Invoke(new EventHandler(OnLogout));
        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            msn.Account = txtEmail.Text;
            msn.Password = txtPassword.Text;
            msn.Login();
        }

        private void menuItem3_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender, e);
        }

        private void menuItem4_Click(object sender, EventArgs e)
        {
            msn.Logout();
        }

        private void tvMSN_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (msn.Logined && (e.Node.Tag != null))
                mmAction.Enabled = (e.Node.Tag is MSNBuddy);
        }

        private void menuItem14_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void menuItem8_Click(object sender, EventArgs e)
        {
            if (msn.Logined && (tvMSN.SelectedNode.Tag != null))
                msn.BlockBuddy((tvMSN.SelectedNode.Tag as MSNBuddy).Email);
        }

        private void menuItem9_Click(object sender, EventArgs e)
        {
            if (msn.Logined && (tvMSN.SelectedNode.Tag != null))
                msn.UnblockBuddy((tvMSN.SelectedNode.Tag as MSNBuddy).Email);
        }

        private void menuItem10_Click(object sender, EventArgs e)
        {
            if (msn.Logined && (tvMSN.SelectedNode.Tag != null))
                msn.DeleteBuddy((tvMSN.SelectedNode.Tag as MSNBuddy).Email);
        }

        private void menuItem12_Click(object sender, EventArgs e)
        {

        }

        private void OnTextMessageReceived(object sender, MSNChatSession chatSession, TextMessage message)
        {
            txtLog.Text = message.Buddy + " Said: " + message.Text + "\r\n" + txtLog.Text;
            sbar.Text = "Receive message from " + message.Buddy;
        }

        private void MSN_OnTextMessageReceived(object sender, MSNChatSession chatSession, TextMessage message)
        {
            this.Invoke(new OnTextMessageEventHandler(OnTextMessageReceived), new object[] { sender, chatSession, message });
        }

        private void MSN_OnError(object sender, string errorMessage)
        {
            this.Invoke(new OnErrorEventHandler(OnLog), new object[] { sender, errorMessage });
        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            msn.Status = ((MSNStatus)cbStatus.SelectedIndex);
        }

        private void menuItem2_Click(object sender, EventArgs e)
        {
            if (tvMSN.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    TextMessage msg = new TextMessage();
                    msg.Buddy = (tvMSN.SelectedNode.Tag as MSNBuddy).Email;
                    msg.Text = frmInput.MessageStr;
                    msn.SendMessage(msg);
                }
            }
        }

        private void menuItem5_Click(object sender, EventArgs e)
        {
            if (tvMSN.SelectedNode != null)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.SetCaption("Offline Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    msn.SendOfflineMessage((tvMSN.SelectedNode.Tag as MSNBuddy).Email,frmInput.MessageStr);
                }
            }
        }

        private void txtNickname_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                if (txtNickname.Text == "") txtNickname.Text = msn.Account;
                msn.Nickname = txtNickname.Text;
            }
        }

        private void pbAvatar_DoubleClick(object sender, EventArgs e)
        {

        }

    }
}